/* Copyright (c) 2005, Carlos Lamas
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

   * Neither the name of the copyright holders nor the names of
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. */


#ifndef _INTTYPES_H_

#define _INTTYPES_H_

#include <stdint.h>



/* Far pointers for memory access >64K */

typedef		int32_t			int_farptr_t;
typedef		uint32_t		uint_farptr_t;



/* 7.8.1 Macros for format specifiers */
/* avr-libc currently does not admit all of them */


#if !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS)


/* fprintf macros for signed integers */

#define		PRId8			"d"
#define		PRIdLEAST8		"d"
#define		PRIdFAST8		"d"

#define		PRIi8			"i"
#define		PRIiLEAST8		"i"
#define		PRIiFAST8		"i"


#if __USING_MINT8


#define		PRId16			"ld"
#define		PRIdLEAST16		"ld"
#define		PRIdFAST16		"ld"

#define		PRIi16			"li"
#define		PRIiLEAST16		"li"
#define		PRIiFAST16		"li"


#if !__HAS_INT64_T__

#define		PRId32			"lld"
#define		PRIdLEAST32		"lld"
#define		PRIdFAST32		"lld"

#define		PRIi32			"lli"
#define		PRIiLEAST32		"lli"
#define		PRIiFAST32		"lli"

#endif	/*  !__HAS_INT64_T__ */


#else	/* !__USING_MINT8 */


#define		PRId16			"d"
#define		PRIdLEAST16		"d"
#define		PRIdFAST16		"d"

#define		PRIi16			"i"
#define		PRIiLEAST16		"i"
#define		PRIiFAST16		"i"


#define		PRId32			"ld"
#define		PRIdLEAST32		"ld"
#define		PRIdFAST32		"ld"

#define		PRIi32			"li"
#define		PRIiLEAST32		"li"
#define		PRIiFAST32		"li"


#endif	/* __USING_MINT8 */


#if __HAS_INT64_T__

#define		PRId64			"lld"
#define		PRIdLEAST64		"lld"
#define		PRIdFAST64		"lld"

#define		PRIi64			"lli"
#define		PRIiLEAST64		"lli"
#define		PRIiFAST64		"lli"

#endif	/* __HAS_INT64_T__ */


#define		PRIdMAX			"lld"
#define		PRIiMAX			"lli"


#define		PRIdPTR			PRId16
#define		PRIiPTR			PRIi16


/* fprintf macros for unsigned integers */

#define		PRIo8			"o"
#define		PRIoLEAST8		"o"
#define		PRIoFAST8		"o"

#define		PRIu8			"u"
#define		PRIuLEAST8		"u"
#define		PRIuFAST8		"u"

#define		PRIx8			"x"
#define		PRIxLEAST8		"x"
#define		PRIxFAST8		"x"

#define		PRIX8			"X"
#define		PRIXLEAST8		"X"
#define		PRIXFAST8		"X"


#if __USING_MINT8


#define		PRIo16			"lo"
#define		PRIoLEAST16		"lo"
#define		PRIoFAST16		"lo"

#define		PRIu16			"lu"
#define		PRIuLEAST16		"lu"
#define		PRIuFAST16		"lu"

#define		PRIx16			"lx"
#define		PRIxLEAST16		"lx"
#define		PRIxFAST16		"lx"

#define		PRIX16			"lX"
#define		PRIXLEAST16		"lX"
#define		PRIXFAST16		"lX"


#if !__HAS_INT64_T__

#define		PRIo32			"llo"
#define		PRIoLEAST32		"llo"
#define		PRIoFAST32		"llo"

#define		PRIu32			"llu"
#define		PRIuLEAST32		"llu"
#define		PRIuFAST32		"llu"

#define		PRIx32			"llx"
#define		PRIxLEAST32		"llx"
#define		PRIxFAST32		"llx"

#define		PRIX32			"llX"
#define		PRIXLEAST32		"llX"
#define		PRIXFAST32		"llX"

#endif	/* !__HAS_INT64_T__ */


#else /* !__USING_MINT8 */


#define		PRIo16			"o"
#define		PRIoLEAST16		"o"
#define		PRIoFAST16		"o"

#define		PRIu16			"u"
#define		PRIuLEAST16		"u"
#define		PRIuFAST16		"u"

#define		PRIx16			"x"
#define		PRIxLEAST16		"x"
#define		PRIxFAST16		"x"

#define		PRIX16			"X"
#define		PRIXLEAST16		"X"
#define		PRIXFAST16		"X"


#define		PRIo32			"lo"
#define		PRIoLEAST32		"lo"
#define		PRIoFAST32		"lo"

#define		PRIu32			"lu"
#define		PRIuLEAST32		"lu"
#define		PRIuFAST32		"lu"

#define		PRIx32			"lx"
#define		PRIxLEAST32		"lx"
#define		PRIxFAST32		"lx"

#define		PRIX32			"lX"
#define		PRIXLEAST32		"lX"
#define		PRIXFAST32		"lX"


#endif	/* __USING_MINT8 */


#if __HAS_INT64_T__

#define		PRIo64			"llo"
#define		PRIoLEAST64		"llo"
#define		PRIoFAST64		"llo"

#define		PRIu64			"llu"
#define		PRIuLEAST64		"llu"
#define		PRIuFAST64		"llu"

#define		PRIx64			"llx"
#define		PRIxLEAST64		"llx"
#define		PRIxFAST64		"llx"

#define		PRIX64			"llX"
#define		PRIXLEAST64		"llX"
#define		PRIXFAST64		"llX"

#endif	/* __HAS_INT64_T__ */


#define		PRIoMAX			"llo"
#define		PRIuMAX			"llu"
#define		PRIxMAX			"llx"
#define		PRIXMAX			"llX"


#define		PRIoPTR			PRIo16
#define		PRIuPTR			PRIu16
#define		PRIxPTR			PRIx16
#define		PRIXPTR			PRIX16



/* The fscanf macros for signed integers */


#if __USING_MINT8


#define		SCNd8			"d"
#define		SCNdLEAST8		"d"
#define		SCNdFAST8		"d"

#define		SCNi8			"i"
#define		SCNiLEAST8		"i"
#define		SCNiFAST8		"i"


#define		SCNd16			"ld"
#define		SCNdLEAST16		"ld"
#define		SCNdFAST16		"ld"

#define		SCNi16			"li"
#define		SCNiLEAST16		"li"
#define		SCNiFAST16		"li"


#if !__HAS_INT64_T__

#define		SCNd32			"lld"
#define		SCNdLEAST32		"lld"
#define		SCNdFAST32		"lld"

#define		SCNi32			"lli"
#define		SCNiLEAST32		"lli"
#define		SCNiFAST32		"lli"

#endif	/* !__HAS_INT64_T__ */


#else	/* !__USING_MINT8 */


#define		SCNd8			"hhd"
#define		SCNdLEAST8		"hhd"
#define		SCNdFAST8		"hhd"

#define		SCNi8			"hhi"
#define		SCNiLEAST8		"hhi"
#define		SCNiFAST8		"hhi"


#define		SCNd16			"d"
#define		SCNdLEAST16		"d"
#define		SCNdFAST16		"d"

#define		SCNi16			"i"
#define		SCNiLEAST16		"i"
#define		SCNiFAST16		"i"


#define		SCNd32			"ld"
#define		SCNdLEAST32		"ld"
#define		SCNdFAST32		"ld"

#define		SCNi32			"li"
#define		SCNiLEAST32		"li"
#define		SCNiFAST32		"li"


#endif	/* __USING_MINT8 */


#if __HAS_INT64_T__

#define		SCNd64			"lld"
#define		SCNdLEAST64		"lld"
#define		SCNdFAST64		"lld"

#define		SCNi64			"lli"
#define		SCNiLEAST64		"lli"
#define		SCNiFAST64		"lli"

#endif


#define		SCNdMAX			"lld"
#define		SCNiMAX			"lli"


#define		SCNdPTR			SCNd16
#define		SCNiPTR			SCNi16


/* The fscanf macros for unsigned integers */


#if __USING_MINT8


#define		SCNo8			"o"
#define		SCNoLEAST8		"o"
#define		SCNoFAST8		"o"

#define		SCNu8			"u"
#define		SCNuLEAST8		"u"
#define		SCNuFAST8		"u"

#define		SCNx8			"x"
#define		SCNxLEAST8		"x"
#define		SCNxFAST8		"x"


#define		SCNo16			"lo"
#define		SCNoLEAST16		"lo"
#define		SCNoFAST16		"lo"

#define		SCNu16			"lu"
#define		SCNuLEAST16		"lu"
#define		SCNuFAST16		"lu"

#define		SCNx16			"lx"
#define		SCNxLEAST16		"lx"
#define		SCNxFAST16		"lx"


#if !__HAS_INT64_T__

#define		SCNo32			"llo"
#define		SCNoLEAST32		"llo"
#define		SCNoFAST32		"llo"

#define		SCNu32			"llu"
#define		SCNuLEAST32		"llu"
#define		SCNuFAST32		"llu"

#define		SCNx32			"llx"
#define		SCNxLEAST32		"llx"
#define		SCNxFAST32		"llx"

#endif	/* !__HAS_INT64_T__ */


#else /* !__USING_MINT8 */


#define		SCNo8			"hho"
#define		SCNoLEAST8		"hho"
#define		SCNoFAST8		"hho"

#define		SCNu8			"hhu"
#define		SCNuLEAST8		"hhu"
#define		SCNuFAST8		"hhu"

#define		SCNx8			"hhx"
#define		SCNxLEAST8		"hhx"
#define		SCNxFAST8		"hhx"


#define		SCNo16			"o"
#define		SCNoLEAST16		"o"
#define		SCNoFAST16		"o"

#define		SCNu16			"u"
#define		SCNuLEAST16		"u"
#define		SCNuFAST16		"u"

#define		SCNx16			"x"
#define		SCNxLEAST16		"x"
#define		SCNxFAST16		"x"


#define		SCNo32			"lo"
#define		SCNoLEAST32		"lo"
#define		SCNoFAST32		"lo"

#define		SCNu32			"lu"
#define		SCNuLEAST32		"lu"
#define		SCNuFAST32		"lu"

#define		SCNx32			"lx"
#define		SCNxLEAST32		"lx"
#define		SCNxFAST32		"lx"


#endif	/* __USING_MINT8 */


#if __HAS_INT64_T__

#define		SCNo64			"llo"
#define		SCNoLEAST64		"llo"
#define		SCNoFAST64		"llo"

#define		SCNu64			"llu"
#define		SCNuLEAST64		"llu"
#define		SCNuFAST64		"llu"

#define		SCNx64			"llx"
#define		SCNxLEAST64		"llx"
#define		SCNxFAST64		"llx"

#endif	/* __HAS_INT64_T__ */


#define		SCNoMAX			"llo"
#define		SCNuMAX			"llu"
#define		SCNxMAX			"llx"


#define		SCNoPTR			SCNo16
#define		SCNuPTR			SCNu16
#define		SCNxPTR			SCNx16



#endif	/* !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS) */


#endif	/* _INTTYPES_H_ */
