/* Copyright (c) 2005, Carlos Lamas
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

   * Neither the name of the copyright holders nor the names of
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. */

/*
 * ISO/IEC 9899:1999  7.18 Integer types <stdint.h>
 */


#ifndef _STDINT_H_

#define _STDINT_H_


/* Helping macro */
#ifndef __CONCAT
#define __CONCATenate(left, right) left ## right
#define __CONCAT(left, right) __CONCATenate(left, right)
#endif


#if __INT_MAX__ == 127
#define __USING_MINT8 1
#endif

#define __HAS_INT32_T__ 1

#if (!__USING_MINT8) || (__GNUC__ < 3) || (__GNUC__ == 3 && __GNUC_MINOR__ < 5)
#define __HAS_INT64_T__ 1
#endif


/* 7.18.1 Integer types */


/* 7.18.1.1 Exact-width integer types */
/* (Integer types having exactly the specified width) */


typedef		signed		int8_t		__attribute__((__mode__(__QI__)));
typedef		unsigned	uint8_t		__attribute__((__mode__(__QI__)));

typedef		signed		int16_t		__attribute__((__mode__(__HI__)));
typedef		unsigned	uint16_t	__attribute__((__mode__(__HI__)));

typedef		signed		int32_t		__attribute__((__mode__(__SI__)));
typedef		unsigned	uint32_t	__attribute__((__mode__(__SI__)));

#if __HAS_INT64_T__
typedef		signed		int64_t		__attribute__((__mode__(__DI__)));
typedef		unsigned	uint64_t	__attribute__((__mode__(__DI__)));
#endif


/* 7.18.1.2 Minimum-width integer types */
/* (Integer types having at least the specified width) */

typedef		int8_t		int_least8_t;
typedef		uint8_t		uint_least8_t;

typedef		int16_t		int_least16_t;
typedef		uint16_t	uint_least16_t;

typedef		int32_t		int_least32_t;
typedef		uint32_t	uint_least32_t;

#if __HAS_INT64_T__
typedef		int64_t		int_least64_t;
typedef		uint64_t	uint_least64_t;
#endif


/* 7.18.1.3 Fastest minimum-width integer types */
/* (Integer types being usually fastest having at least the specified width) */

typedef		int8_t		int_fast8_t;
typedef		uint8_t		uint_fast8_t;

typedef		int16_t		int_fast16_t;
typedef		uint16_t	uint_fast16_t;

typedef		int32_t		int_fast32_t;
typedef		uint32_t	uint_fast32_t;

#if __HAS_INT64_T__
typedef		int64_t		int_fast64_t;
typedef		uint64_t	uint_fast64_t;
#endif


/* 7.18.1.4 Integer types capable of holding object pointers */

typedef		int16_t		intptr_t;
typedef		uint16_t	uintptr_t;


/* 7.18.1.5 Greatest-width integer types */
/* (Types designating integer data capable of representing any value of
any integer type in the corresponding signed or unsigned category) */

#if __HAS_INT64_T__
typedef		int64_t		intmax_t;
typedef		uint64_t	uintmax_t;
#else
typedef		int32_t		intmax_t;
typedef		uint32_t	uintmax_t;
#endif


/* 7.18.2 Limits of specified-width integer types */
/* C++ implementations should define these macros only when
   __STDC_LIMIT_MACROS is defined before <stdint.h> is included */


#if !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS)


/* 7.18.2.1 Limits of exact-width integer types */

#define		INT8_MAX			0x7f
#define		INT8_MIN			(-INT8_MAX - 1)
#define		UINT8_MAX			(__CONCAT(INT8_MAX, U) * 2U + 1U)

#if __USING_MINT8

#define		INT16_MAX			0x7fffL
#define		INT16_MIN			(-INT16_MAX - 1L)
#define		UINT16_MAX			(__CONCAT(INT16_MAX, U) * 2UL + 1UL)

#define		INT32_MAX			0x7fffffffLL
#define		INT32_MIN			(-INT32_MAX - 1LL)
#define		UINT32_MAX			(__CONCAT(INT32_MAX, U) * 2ULL + 1ULL)

#else	/* !__USING_MINT8 */

#define		INT16_MAX			0x7fff
#define		INT16_MIN			(-INT16_MAX - 1)
#define		UINT16_MAX			(__CONCAT(INT16_MAX, U) * 2U + 1U)

#define		INT32_MAX			0x7fffffffL
#define		INT32_MIN			(-INT32_MAX - 1L)
#define		UINT32_MAX			(__CONCAT(INT32_MAX, U) * 2UL + 1UL)

#endif	/* __USING_MINT8 */

#if __HAS_INT64_T__
#define		INT64_MAX			0x7fffffffffffffffLL
#define		INT64_MIN			(-INT64_MAX - 1LL)
#define		UINT64_MAX			(__CONCAT(INT64_MAX, U) * 2ULL + 1ULL)
#endif


/* 7.18.2.2 Limits of minimum-width integer types */

#define		INT_LEAST8_MAX		INT8_MAX
#define		INT_LEAST8_MIN		INT8_MIN
#define		UINT_LEAST8_MAX		UINT8_MAX

#define		INT_LEAST16_MAX		INT16_MAX
#define		INT_LEAST16_MIN		INT16_MIN
#define		UINT_LEAST16_MAX	UINT16_MAX

#define		INT_LEAST32_MAX		INT32_MAX
#define		INT_LEAST32_MIN		INT32_MIN
#define		UINT_LEAST32_MAX	UINT32_MAX

#if __HAS_INT64_T__
#define		INT_LEAST64_MAX		INT64_MAX
#define		INT_LEAST64_MIN		INT64_MIN
#define		UINT_LEAST64_MAX	UINT64_MAX
#endif


/* 7.18.2.3 Limits of fastest minimum-width integer types */

#define		INT_FAST8_MAX		INT8_MAX
#define		INT_FAST8_MIN		INT8_MIN
#define		UINT_FAST8_MAX		UINT8_MAX

#define		INT_FAST16_MAX		INT16_MAX
#define		INT_FAST16_MIN		INT16_MIN
#define		UINT_FAST16_MAX		UINT16_MAX

#define		INT_FAST32_MAX		INT32_MAX
#define		INT_FAST32_MIN		INT32_MIN
#define		UINT_FAST32_MAX		UINT32_MAX

#if __HAS_INT64_T__
#define		INT_FAST64_MAX		INT64_MAX
#define		INT_FAST64_MIN		INT64_MIN
#define		UINT_FAST64_MAX		UINT64_MAX
#endif


/* 7.18.2.4 Limits of integer types capable of holding object pointers */

#define		INTPTR_MAX			INT16_MAX
#define		INTPTR_MIN			INT16_MIN
#define		UINTPTR_MAX			UINT16_MAX


/* 7.18.2.5 Limits of greatest-width integer types */

#if __HAS_INT64_T__
#define		INTMAX_MAX			INT64_MAX
#define		INTMAX_MIN			INT64_MIN
#define		UINTMAX_MAX			UINT64_MAX
#else
#define		INTMAX_MAX			INT32_MAX
#define		INTMAX_MIN			INT32_MIN
#define		UINTMAX_MAX			UINT32_MAX
#endif


#endif /* !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS) */




/* 7.18.3 Limits of other integer types */
/* C++ implementations should define these macros only when
   __STDC_LIMIT_MACROS is defined before <stdint.h> is included */


#if !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS)


/* Limits of ptrdiff_t */

#define		PTRDIFF_MAX			INT16_MAX
#define		PTRDIFF_MIN			INT16_MIN


/* Limits of sig_atomic_t */
/* signal.h is currently not implemented (not avr/signal.h) */
/* Can the AVR architecture allow atomic operations in one data type?? */
/* #define		SIG_ATOMIC_MAX */
/* #define		SIG_ATOMIC_MIN */


/* Limit of size_t */
#define		SIZE_MAX			(__CONCAT(INT16_MAX, U))


/* Limits of wchar_t */
/* wchar.h is currently not implemented */
/* #define		WCHAR_MAX */
/* #define		WCHAR_MIN */


/* Limits of wint_t */
/* wchar.h is currently not implemented */
/* #define		WINT_MAX */
/* #define		WINT_MIN */


#endif /* !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS) */




/* 7.18.4 Macros for integer constants */
/* C++ implementations should define these macros only when
   __STDC_CONSTANT_MACROS is defined before <stdint.h> is included */

#if !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS)

/* These definitions are valid for integer constants without suffix and 
   for macros defined as integer constant without suffix */

/* 7.18.4.1 Macros for minimum-width integer constants */

#define		INT8_C(value)		((int8_t) value)
#define		UINT8_C(value)		((uint8_t) __CONCAT(value, U))

#if __USING_MINT8

#define		INT16_C(value)		__CONCAT(value, L)
#define		UINT16_C(value)		__CONCAT(value, UL)

#if __HAS_INT64_T__
#define		INT32_C(value)		((int32_t) __CONCAT(value, LL))
#define		UINT32_C(value)		((uint32_t) __CONCAT(value, ULL))
#else
#define		INT32_C(value)		__CONCAT(value, LL)
#define		UINT32_C(value)		__CONCAT(value, ULL)
#endif

#else	/* !__USING_MINT8 */

#define		INT16_C(value)		value
#define		UINT16_C(value)		__CONCAT(value, U)

#define		INT32_C(value)		__CONCAT(value, L)
#define		UINT32_C(value)		__CONCAT(value, UL)

#endif	/* __USING_MINT8 */

#if __HAS_INT64_T__
#define		INT64_C(value)		__CONCAT(value, LL)
#define		UINT64_C(value)		__CONCAT(value, ULL)
#endif


/* 7.18.4.2 Macros for greatest-width integer constants */

#define		INTMAX_C(value)		__CONCAT(value, LL)
#define		UINTMAX_C(value)	__CONCAT(value, ULL)


#endif	/* !defined(__cplusplus) || defined(__STDC_LIMIT_MACROS) */


#endif		/* _STDINT_H_ */
